@@ -47,6 +47,7 @@ Created 3/14/1997 Heikki Tuuri
 #include "handler.h"
 #include "ha_innodb.h"
 #include "fil0fil.h"
+#include "debug_sync.h"
 #include <mysql/service_thd_mdl.h>
 
 /*************************************************************************
@@ -615,6 +616,8 @@ row_purge_del_mark(
       const auto type= node->index->type;
       if (type & (DICT_FTS | DICT_CORRUPT))
         continue;
+      if (node->index->online_status > ONLINE_INDEX_CREATION)
+        continue;
       if (UNIV_UNLIKELY(DICT_VIRTUAL & type) && !node->index->is_committed() &&
           node->index->has_new_v_col())
         continue;
@@ -629,7 +632,17 @@ row_purge_del_mark(
     mem_heap_free(heap);
   }
 
-  return row_purge_remove_clust_if_poss(node);
+  bool result= row_purge_remove_clust_if_poss(node);
+
+#ifdef ENABLED_DEBUG_SYNC
+  DBUG_EXECUTE_IF("enable_row_purge_del_mark_exit_sync_point",
+                  debug_sync_set_action
+                  (current_thd,
+                   STRING_WITH_LEN("now SIGNAL row_purge_del_mark_finished"));
+                  );
+#endif
+
+  return result;
 }
 
 void purge_sys_t::wait_SYS()
@@ -767,6 +780,11 @@ row_purge_upd_exist_or_extern_func(
 			continue;
 		}
 
+		if (node->index->online_status
+		    > ONLINE_INDEX_CREATION) {
+			continue;
+		}
+
 		if (row_upd_changes_ord_field_binary(node->index, node->update,
 						     thr, NULL, NULL)) {
 			/* Build the older version of the index entry */
@@ -843,6 +861,9 @@ skip_secondaries:
 				   buf_page_get(page_id_t(rseg.space->id,
 							  page_no),
 						0, RW_X_LATCH, &mtr)) {
+				block->page.set_accessed();
+				buf_page_make_young_if_needed(&block->page);
+
 				byte* data_field = block->page.frame
 					+ offset + internal_offset;
 
